function [net,objective]  = finetune_first_hmm(net,posdata,negdata,varargin)%,image,gtbox)%(net,pos_data,neg_data,varargin)
% MDNET_FINETUNE_HNM   
% Train a CNN by SGD, with hard minibatch mining.
%
% modified from cnn_train() in the MatConvNet library.
% Hyeonseob Nam, 2015
%

opts.useGpu = true;
opts.conserveMemory = true ;
opts.sync = true ;

opts.maxiter = 300;
opts.learningRate = 0.001;
opts.weightDecay = 0.0005 ;
opts.momentum = 0.9 ;

opts.batchSize_hnm = 32;
opts.batchAcc_hnm = 4;

opts.batchSize = 8;
opts.batch_pos = 2;
opts.batch_neg = 6;

opts = vl_argparse(opts, varargin) ;
opts.batchSize = 8;
opts.batch_pos = 2;
opts.batch_neg = 6;
batchsize=8;
% -------------------------------------------------------------------------
%                                                    Network initialization
% -------------------------------------------------------------------------

for i=1:numel(net.layers)
    if strcmp(net.layers{i}.type,'conv')
        net.layers{i}.filtersMomentum = zeros(size(net.layers{i}.filters), ...
            class(net.layers{i}.filters)) ;
        net.layers{i}.biasesMomentum = zeros(size(net.layers{i}.biases), ...
            class(net.layers{i}.biases)) ; %#ok<*ZEROLIKE>
        
        if opts.useGpu
            net.layers{i}.filtersMomentum = gpuArray(net.layers{i}.filtersMomentum);
            net.layers{i}.biasesMomentum = gpuArray(net.layers{i}.biasesMomentum);
        end
    end
end

%% initilizing
if opts.useGpu
    one = gpuArray(single(1)) ;
else
    one = single(1) ;
end
res = [] ;

n_pos = min(500, size(posdata, 4)); %size(pos_data,4);
n_neg = 5000; %(neg_data,4);
train_pos_cnt = 0;
train_neg_cnt = 0;

% extract positive batches
train_pos = [];
remain = opts.batch_pos*opts.maxiter;
while(remain>0)
    if(train_pos_cnt==0)
        train_pos_list = randperm(n_pos)';
    end
    train_pos = cat(1,train_pos,...
        train_pos_list(train_pos_cnt+1:min(end,train_pos_cnt+remain)));
    train_pos_cnt = min(length(train_pos_list),train_pos_cnt+remain);
    train_pos_cnt = mod(train_pos_cnt,length(train_pos_list));
    remain = opts.batch_pos*opts.maxiter-length(train_pos);
end

% extract negative batches
train_neg = [];
remain = opts.batchSize_hnm*opts.batchAcc_hnm*opts.maxiter;
while(remain>0)
    if(train_neg_cnt==0)
        train_neg_list = randperm(n_neg)';
    end
    train_neg = cat(1,train_neg,...
        train_neg_list(train_neg_cnt+1:min(end,train_neg_cnt+remain)));
    train_neg_cnt = min(length(train_neg_list),train_neg_cnt+remain);
    train_neg_cnt = mod(train_neg_cnt,length(train_neg_list));
    remain = opts.batchSize_hnm*opts.batchAcc_hnm*opts.maxiter-length(train_neg);
end

% learning rate
lr = opts.learningRate ;

% for saving positives
poss = [];

% for saving hard negatives
hardnegs = [];

% objective fuction
objective = zeros(1,opts.maxiter);

%% training on training set
% fprintf('\n');
datestr(now,21)
for t=1:opts.maxiter
     
%     fprintf('\ttraining batch %3d of %3d ... ', t, opts.maxiter) ;
    iter_time = tic ;
    datestr(now,21)
    % ----------------------------------------------------------------------
    % hard negative mining
    % ----------------------------------------------------------------------
    score_hneg = zeros(opts.batchSize_hnm*opts.batchAcc_hnm,1);
    hneg_start = opts.batchSize_hnm*opts.batchAcc_hnm*(t-1);
    for h=1:opts.batchAcc_hnm
        indexs = train_neg(hneg_start+(h-1)*opts.batchSize_hnm+1:hneg_start+h*opts.batchSize_hnm);
        batch = negdata(:,:,:,indexs);
        
%         batch = neg_data(:,:,:,...
%             );
        if opts.useGpu
            batch = gpuArray(batch);
        end
        
        % backprop
        net.layers{end}.class = ones(batchsize,1,'single') ;
        partScore = [];
        for indexBatch=1:size(batch,4)/batchsize
             res = vl_simplenn(net, batch(:,:,:,(indexBatch-1)*batchsize+1:indexBatch*batchsize), [], res, ...
            'disableDropout', true, ...
            'conserveMemory', opts.conserveMemory, ...
            'sync', opts.sync) ;
            partScore = [partScore squeeze(gather(res(end-1).x(1,1,2,:)))];
            res=[];
        end
        
        score_hneg((h-1)*opts.batchSize_hnm+1:h*opts.batchSize_hnm) = partScore;
    end
    [~,ord] = sort(score_hneg,'descend');
%     hnegs = train_neg(hneg_start+ord(1:opts.batch_neg));
    
%     im_hneg = neg_data(:,:,:,hnegs);
    nbatch = negdata(:,:,:,train_neg(hneg_start+ord(1:opts.batch_neg))) ;
%    nbatch =zeros(224,224,3,opt.batch_pos);
%    hnegs = train_neg(hneg_start+ord(1:opts.batch_neg));
%    for nNeg=1:opt.batch_neg   
%        index = hnegs(nNeg);
%       index = train_neg(hneg_start+ord(1:opts.batch_neg));%train_neg((t-1)*opts.batch_pos+nNeg);
%        nbatch(:,:,:,nNeg) = readFromHdf5(negfile,'/neg',[1 1 1 index],[224 224 3 1]);
%    end
    
%     fprintf('hnm: %d/%d, ', opts.batch_neg, opts.batchSize_hnm*opts.batchAcc_hnm) ;
%     hardnegs = [hardnegs; hnegs];
    
    % ----------------------------------------------------------------------
    % get next image batch and labels
    % ----------------------------------------------------------------------
    poss = [poss; train_pos((t-1)*opts.batch_pos+1:t*opts.batch_pos)];
    
    batch = cat(4,posdata(:,:,:,train_pos((t-1)*opts.batch_pos+1:t*opts.batch_pos)),nbatch);    
    
%    batch =zeros(224,224,3,opt.batch_pos);
%    for nPos=1:opt.batch_pos
%        index = train_pos((t-1)*opts.batch_pos+nPos);
%        batch(:,:,:,nPos) = readFromHdf5(posfile,'/pos',[1 1 1 index],[224 224 3 1]);
%    end
%     batch = cat(4,batch,nbatch);
%     batch = cat(4,readFromHdf5('sample.h5','pos',[1 1 1 (t-1)*opt.batch_pos+1],[224 224 3 opt.batch_pos]),im_hneg);
    %cat(4,pos_data(:,:,:,train_pos((t-1)*opts.batch_pos+1:t*opts.batch_pos)),im_hneg);
    
    labels = [2*ones(opts.batch_pos,1,'single');ones(opts.batch_neg,1,'single')];
    if opts.useGpu
        batch = gpuArray(batch) ;
    end
    res = [];
    % backprop
    net.layers{end}.class = labels ;
    res = vl_simplenn(net, batch, one, res, ...
        'conserveMemory', opts.conserveMemory, ...
        'sync', opts.sync) ;
    
    % gradient step
    for l=1:numel(net.layers)
        if ~strcmp(net.layers{l}.type, 'conv'), continue ; end
        
        net.layers{l}.filtersMomentum = ...
            opts.momentum * net.layers{l}.filtersMomentum ...
            - (lr * net.layers{l}.filtersLearningRate) * ...
            (opts.weightDecay * net.layers{l}.filtersWeightDecay) * net.layers{l}.filters ...
            - (lr * net.layers{l}.filtersLearningRate) / opts.batchSize * res(l).dzdw{1} ;
        
        net.layers{l}.biasesMomentum = ...
            opts.momentum * net.layers{l}.biasesMomentum ...
            - (lr * net.layers{l}.biasesLearningRate) * ....
            (opts.weightDecay * net.layers{l}.biasesWeightDecay) * net.layers{l}.biases ...
            - (lr * net.layers{l}.biasesLearningRate) / opts.batchSize * res(l).dzdw{2} ;
        
        net.layers{l}.filters = net.layers{l}.filters + net.layers{l}.filtersMomentum ;
        net.layers{l}.biases = net.layers{l}.biases + net.layers{l}.biasesMomentum ;
    end
    
    % print information
    objective(t) = gather(res(end).x)/opts.batchSize ;
    iter_time = toc(iter_time);
      fprintf('objective %.3f, %.2f s\n', mean(objective(1:t)), iter_time) ;
    
end % next batch
