function cnnFeature = getCnnFeature(img, featureRect, layers)
% 'type' can be 'normalTrain', 'occTrain', 'normalDetect' or 'occDetect'
global enableGPU
global net
global validChannelIdx

adjustNetImageSize(featureRect);

imgSize = size(img);
rect = featureRect.rect;
rows = rect(1) : (rect(1) + rect(3) - 1);
cols = rect(2) : (rect(2) + rect(4) - 1);
rows(rows < 1) = 1;  cols(cols < 1) = 1;
rows(rows > imgSize(1)) = imgSize(1);
cols(cols > imgSize(2)) = imgSize(2);
im = img(rows, cols, :);
imgPatch = single(im);
imgPatch = imResample(imgPatch, net.normalization.imageSize(1:2), 'bilinear');
imgPatch = imgPatch - net.normalization.averageImage;
if enableGPU
    imgPatch = gpuArray(imgPatch);
end

res = vl_simplenn(net, imgPatch);

mapValues = cell(length(layers), 1);
for l = 1 : length(layers)
    if enableGPU
        channels = double(gather(res(layers(l)).x));
    else
        channels = double(res(layers(l)).x);
    end

    if isempty(validChannelIdx)
        validChannelIdx = containers.Map('KeyType', 'double', 'ValueType', 'any');
    end
    if ~isKey(validChannelIdx, layers(l))
        insertNewValidChannelIdx(channels, layers(l));
    end
    channels = channels(:, :, validChannelIdx(layers(l)));
    channels = normalizeChannels(channels);
    feature.channels = imResample(channels, rect(3:4));
    feature.rect = rect;
    feature.im = im;
    mapValues{l} = feature;
end

cnnFeature = containers.Map(layers, mapValues);

end

function adjustNetImageSize(featureRect)
global net
global averageImageMap

if isempty(averageImageMap)
    averageImageMap = containers.Map('KeyType', 'char', 'valueType', 'any');
    averageImageMap('original') = net.normalization.averageImage;
end

if ~isKey(averageImageMap, featureRect.type)
    oriSize = size(averageImageMap('original'));
    newSize = round(oriSize(1:2) .* featureRect.padding ./ 16) * 16;
    newImage = imResample(averageImageMap('original'), newSize);
    averageImageMap(featureRect.type) = newImage;
    net.normalization.averageImage = newImage;
    net.normalization.imageSize = size(newImage);
else
    net.normalization.averageImage = averageImageMap(featureRect.type);
    net.normalization.imageSize = size(net.normalization.averageImage);
end
end

function channels = normalizeChannels(channels)
channels = channels ./ max(channels(:));
% channels(channels < 0.05) = 0;
end

function insertNewValidChannelIdx(channels, l)
global validChannelIdx
idx = [];
for i = 1 : size(channels, 3)
    c = channels(:,:,i);
    if any(c(:))
        idx = [idx, i];
    end
end
validChannelIdx(l) = idx;
end
