function MkcfResults = initTracker(img, rect, Params)
clear global validChannelIdx;
clear global paddingParams;
clear global averageImageMap;
clear global net;
clear global normalAverageImage
clear global occAverageImage

global enableGPU
enableGPU = true;

% clear mex;

% addpath('external/matconvnet/matlab');
% addpath('external/matconvnet/matlab/mex');
% addpath('external/matconvnet/matlab/xtest');

% global net

lambda1 = Params.lambda1;
lambda2 = Params.lambda2;
eta = Params.eta;
scaleSensitivity = Params.scaleSensitivity;
outputSigmaFactor = Params.outputSigmaFactor;
cellSize = Params.cellSize;
paddingSets = Params.paddingSets;
visualization = Params.visualization;
scaleDetection = Params.scaleDetection;
outputIteration = Params.outputIteration;
allLayers = Params.allLayers;
winRectLayer = Params.winRectLayer;

initial_net_with_finetunning(img, rect); % net is global

if size(img, 3) == 1
    img = repmat(img, [1,1,3]);
end
objRect = rect([2,1,4,3]);
imgSize = size(img);
if sqrt(prod(objRect(3:4))) >= 150 || prod(imgSize(1:2)) > 400000
    resampleRactor = 0.5;
else
    resampleRactor = 1;
end

objRect = floor(objRect * resampleRactor);
init_paddingParams(paddingSets, objRect(3:4), size(img));

% winRect
winRect = changeRectPadding(objRect, 'originalRect', 'winRect');
initWinRect = winRect;
normalizedWinRectSize = initWinRect(3:4);
outputSigma = sqrt(prod(objRect(3:4))) * outputSigmaFactor / cellSize;
winRectY_f = fft2(gaussian_shaped_labels(outputSigma, floor(normalizedWinRectSize / cellSize)));


% scaleRect
if scaleDetection == 1
    scaleWinRect = changeRectPadding(winRect, 'winRect', 'scaleRect');
    normalizedScaleRectSize = scaleWinRect(3:4);

    scaleOutputSigma = sqrt(prod(objRect(3:4))) / 15 / cellSize;
    y_scale_f = fft2(fftshift(gaussian_shaped_labels(scaleOutputSigma, floor(normalizedScaleRectSize / cellSize))));
end

if resampleRactor < 1
    img = imresize(img, resampleRactor);
end
imgSize = size(img);

if prod(winRect(3:4)) / (paddingSets.padding.generic ^ 2) >= prod(imgSize(1:2))
    winRect = changeRectPadding(winRect, [0.5, 0.5]);
end


% training
occBigRect = changeRectPadding(winRect, 'winRect', 'occBigRect');
trainType = 'normalTrain';
cnnFeatureRect = calcCnnFeatureRect(occBigRect, winRect, trainType);
cnnFeature = getCnnFeature(img, cnnFeatureRect, allLayers);

% winRectSample
winRectPatchs = extractPatchFromCnnFeature(cnnFeature, winRectLayer, winRect, normalizedWinRectSize, cellSize, 1, 'hannwin', []);
[newWinRectAlpha_f, newWinRectD] = trainModel(winRectPatchs, winRectY_f, lambda1, lambda2, 0.03, 'linear', []);
winRectSamples = winRectPatchs;
winRectAlpha_f = newWinRectAlpha_f;
winRectD = newWinRectD;

% disp(['winRectD: ' num2str(winRectD(1)) ' ' num2str(winRectD(2))]);

% scaleSample
if scaleDetection == 1
    scaleRect = changeRectPadding(winRect, 'winRect', 'scaleRect');   
    scalePatchs = extractScaleFeature(img, 'both', scaleRect, normalizedScaleRectSize, cellSize, 9, 1, 'hannwin', []);
    [newScaleAlpha_f, newScaleD] = trainModel(scalePatchs, y_scale_f, 0.01, lambda2, 0.03, 'gaussian', []);
    scaleSamples = scalePatchs;
    scaleAlpha_f = newScaleAlpha_f;
    scaleD = newScaleD;
end

objRect = changeRectPadding(winRect, 'winRect', 'originalRect');
realRect = objRect ./ resampleRactor;

MkcfResults.resampleRactor = resampleRactor;
MkcfResults.winRect = winRect;
MkcfResults.normalizedWinRectSize = normalizedWinRectSize;
MkcfResults.winRectY_f = winRectY_f;
MkcfResults.normalizedScaleRectSize = normalizedScaleRectSize;
MkcfResults.y_scale_f = y_scale_f;
MkcfResults.occBigRect = occBigRect;
MkcfResults.initWinRect = initWinRect;

MkcfResults.winRectSamples = winRectSamples;
MkcfResults.winRectAlpha_f = winRectAlpha_f;
MkcfResults.winRectD = winRectD;

MkcfResults.scaleSamples = scaleSamples;
MkcfResults.scaleAlpha_f = scaleAlpha_f;
MkcfResults.scaleD = scaleD;

MkcfResults.realRect = realRect;

end