#include "mex.h"
#include <stdlib.h>
#include <vector>
#include <queue>
#include <unordered_set>
#include <string>

using namespace std;

/*
struct Point {
	double val;
	int r;
	int c;
	Point(double _v, int _r, int _c): val(_v), r(_r), c(_c) {}
};

bool operator< (const Point &a, const Point &b) {
	return a.val < b.val;
}

inline int myHash(int r, int c, int w) {
	return r * w + c;
}

vector<pair<int,int>> findMaxPoint(double **matrix, int h, int w, int sideLen) {
	// TODO: sideLen must be odd!
	vector<pair<int,int>> res;
	if (h < sideLen || w < sideLen)
		return res;
	priority_queue<Point> Q;
	// initialize
	for (int r = 0; r < sideLen; ++r) {
		for (int c = 0; c < sideLen; ++c)
			Q.push(Point(matrix[r][c], r, c));
	}
	int center_r = sideLen / 2, center_c = sideLen / 2;
	if (Q.top().r == center_r && Q.top().c == center_c)
		res.push_back(make_pair(Q.top().r, Q.top().c));
	unordered_set<int> needDel;
	int r = sideLen - 1;
	while (true) {
		if (!((r - sideLen + 1) & 0x01)) {
			// move right
			for (int c = sideLen; c < w; ++c) {
				++center_c;
				for (int rr = r - sideLen + 1; rr <= r; ++rr) {
					Q.push(Point(matrix[rr][c], rr, c));
					needDel.insert(myHash(rr, c - sideLen, w));
				}
				while (needDel.count(myHash(Q.top().r, Q.top().c, w))) {
					needDel.count(myHash(Q.top().r, Q.top().c, w));
					Q.pop();
				}
				if (Q.top().r == center_r && Q.top().c == center_c)
					res.push_back(make_pair(Q.top().r, Q.top().c));
			}
		} else {
			// move left
			for (int c = w - sideLen - 1; c >= 0; --c) {
				--center_c;
				for (int rr = r - sideLen + 1; rr <= r; ++rr) {
					Q.push(Point(matrix[rr][c], rr, c));
					needDel.insert(myHash(rr, c + sideLen, w));
				}
				while (needDel.count(myHash(Q.top().r, Q.top().c, w))) {
					needDel.erase(myHash(Q.top().r, Q.top().c, w));
					Q.pop();
				}
				if (Q.top().r == center_r && Q.top().c == center_c)
					res.push_back(make_pair(Q.top().r, Q.top().c));
			}
		}
		// move down
		++r;
		++center_r;
		if (r >= w)
			break;
		if ((r - sideLen + 1) & 0x01) {
			for (int cc = w - 1; cc >= w - sideLen; --cc) {
				Q.push(Point(matrix[r][cc], r, cc));
				needDel.insert(myHash(r - sideLen, cc, w));
			}
		} else {
			for (int cc = 0; cc < sideLen; ++cc) {
				Q.push(Point(matrix[r][cc], r, cc));
				needDel.insert(myHash(r - sideLen, cc, w));
			}
		}
		while (needDel.count(myHash(Q.top().r, Q.top().c, w))) {
			needDel.erase(myHash(Q.top().r, Q.top().c, w));
			Q.pop();
		}
		if (Q.top().r == center_r && Q.top().c == center_c)
			res.push_back(make_pair(Q.top().r, Q.top().c));
	}
	return res;
}
*/

vector<pair<int,int>> findMaxPoint(double **matrix, int h, int w, int sideLen) {
	vector<pair<int,int>> res;
	if (h < sideLen || w < sideLen)
		return res;
	for (int r = sideLen / 2; r < h - sideLen / 2; ++r) {
		for (int c = sideLen / 2; c < w - sideLen / 2; ++c) {
			int centerValue = matrix[r][c];
			bool isMax = true;
			for (int rr = r - (sideLen / 2); rr <= r + (sideLen / 2); ++rr) {
				for (int cc = c - (sideLen / 2); cc <= c + sideLen / 2; ++cc) {
					if (rr == r && cc == c)
						continue;
					if (matrix[rr][cc] >= matrix[r][c])
						isMax = false;
					if (!isMax)
						break;
				}
				if (!isMax)
					break;
			}
			if (isMax)
				res.push_back(make_pair(r, c));
		}
	}
	return res;
}


void loadMatrixFromMatlab(const mxArray *mxMatrix, double **matrix) {
	double *scrPtr = mxGetPr(mxMatrix);
	int height = mxGetM(mxMatrix); 
	int width = mxGetN(mxMatrix);
	for (int r = 0; r < height; ++r) {
		for (int c = 0; c < width; ++c) {
			matrix[r][c] = scrPtr[r + c * height];
		}
	}
	return;
}

void outputResToMatlab(const vector<pair<int,int>> &points, mxArray *mxMatrix) {
	double *outPtr = mxGetPr(mxMatrix);
	for (int c = 0; c < points.size(); ++c) {
		outPtr[0 + c * 2] = (double)points[c].first + 1;
		outPtr[1 + c * 2] = (double)points[c].second + 1;
	}
	return;
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
	// prhs[0]: response map     prhs[1]: side length of sub max
	int height = mxGetM(prhs[0]);
	int width = mxGetN(prhs[0]);
	int sideLen = (int)(*mxGetPr(prhs[1])) | 0x01;
	double **response = new double*[height];
	for (int r = 0; r < height; ++r)
		response[r] = new double[width];
	loadMatrixFromMatlab(prhs[0], response);
	vector<pair<int,int>> points = findMaxPoint(response, height, width, sideLen);
	plhs[0] = mxCreateDoubleMatrix(2, points.size(), mxREAL);
	outputResToMatlab(points, plhs[0]);
	for (int r = 0; r < height; ++r)
		delete[] response[r];
	delete[] response;
	return;
}