function MkcfResults = trackFrame(img, MkcfResults, Params, frame)

lambda1 = Params.lambda1;
lambda2 = Params.lambda2;
eta = Params.eta;
scaleSensitivity = Params.scaleSensitivity;
outputSigmaFactor = Params.outputSigmaFactor;
cellSize = Params.cellSize;
paddingSets = Params.paddingSets;
scaleDetection = Params.scaleDetection;
outputIteration = Params.outputIteration;
allLayers = Params.allLayers;
winRectLayer = Params.winRectLayer;

resampleRactor = MkcfResults.resampleRactor;
winRect = MkcfResults.winRect;
normalizedWinRectSize = MkcfResults.normalizedWinRectSize;
winRectY_f = MkcfResults.winRectY_f;
normalizedScaleRectSize = MkcfResults.normalizedScaleRectSize;
y_scale_f = MkcfResults.y_scale_f;
occBigRect = MkcfResults.occBigRect;
initWinRect = MkcfResults.initWinRect;

winRectSamples = MkcfResults.winRectSamples;
winRectAlpha_f = MkcfResults.winRectAlpha_f;
winRectD = MkcfResults.winRectD;

scaleSamples = MkcfResults.scaleSamples;
scaleAlpha_f = MkcfResults.scaleAlpha_f;
scaleD = MkcfResults.scaleD;

if size(img, 3) == 1
    img = repmat(img, [1,1,3]);
end
    
if resampleRactor < 1
    img = imresize(img, resampleRactor);
end
imgSize = size(img);
    
detectType = 'normalDetect';
cnnFeatureRect = calcCnnFeatureRect(occBigRect, winRect, detectType);
cnnFeature = getCnnFeature(img, cnnFeatureRect, allLayers);

winRectPatchs = extractPatchFromCnnFeature(cnnFeature, winRectLayer, winRect, normalizedWinRectSize, cellSize, 1, 'hannwin', []);
resizeRatio = sqrt(prod(winRect(3:4)) / prod(normalizedWinRectSize));
response = detect(winRectSamples, winRectAlpha_f, winRectD, winRectPatchs, 'linear', []);

[maxR, maxC] = find(response == max(response(:)), 1);
% sideLength = ceil(sqrt(prod(normalizedWinRectSize / cellSize)) * outputSigmaFactor * 2);
% [candPoints, maxR, maxC] = findCandPoints(response, sideLength);


if  outputIteration == 1
    sz = floor(normalizedWinRectSize / cellSize);
    ky = circshift(-floor((sz(1) - 1)/2) : ceil((sz(1) - 1)/2), [1, -floor((sz(1) - 1)/2)]);
    kx = circshift(-floor((sz(2) - 1)/2) : ceil((sz(2) - 1)/2), [1, -floor((sz(2) - 1)/2)])';
    [bestDr, bestDc, ~] = resp_newton(response, fft2(response), 5, ky, kx, sz, maxR, maxC);
    winRect(1:2) = winRect(1:2) + round(resizeRatio * cellSize * [bestDr, bestDc]);
else
    [bestDr, bestDc] = correctLoc(maxR, maxC, size(response));
    winRect(1:2) = winRect(1:2) + round(resizeRatio * cellSize * [bestDr - 1, bestDc - 1]);
end

if scaleDetection == 1 && mod(frame, 2) == 1 && (frame - 1) >= 5
    winRect = searchScale(img, winRect, scaleSamples, scaleAlpha_f, scaleD, initWinRect, normalizedScaleRectSize, scaleSensitivity, cellSize);
end

if prod(winRect(3:4)) / (paddingSets.padding.generic ^ 2) >= prod(imgSize(1:2))
    winRect = changeRectPadding(winRect, [0.5, 0.5]);
end

% training
occBigRect = changeRectPadding(winRect, 'winRect', 'occBigRect');
trainType = 'normalTrain';

cnnFeatureRect = calcCnnFeatureRect(occBigRect, winRect, trainType);
cnnFeature = getCnnFeature(img, cnnFeatureRect, allLayers);

% winRectSample
winRectPatchs = extractPatchFromCnnFeature(cnnFeature, winRectLayer, winRect, normalizedWinRectSize, cellSize, 1, 'hannwin', []);
[newWinRectAlpha_f, newWinRectD] = trainModel(winRectPatchs, winRectY_f, lambda1, lambda2, 0.03, 'linear', []);
eta_ad = eta;
winRectSamples = updateSamples(winRectSamples, winRectPatchs, eta_ad);
[winRectAlpha_f, winRectD] = updateMod(winRectAlpha_f, winRectD, newWinRectAlpha_f, newWinRectD, eta_ad);

% scaleSample
if scaleDetection == 1
    scaleRect = changeRectPadding(winRect, 'winRect', 'scaleRect');   
    scalePatchs = extractScaleFeature(img, 'both', scaleRect, normalizedScaleRectSize, cellSize, 9, 1, 'hannwin', []);
    [newScaleAlpha_f, newScaleD] = trainModel(scalePatchs, y_scale_f, 0.01, lambda2, 0.03, 'gaussian', []);
    eta_ad = eta;
    scaleSamples = updateSamples(scaleSamples, scalePatchs, eta_ad * 1);
    [scaleAlpha_f, scaleD] = updateMod(scaleAlpha_f, scaleD, newScaleAlpha_f, newScaleD, eta_ad * 1);
end

objRect = changeRectPadding(winRect, 'winRect', 'originalRect');
realRect = objRect ./ resampleRactor;

% MkcfResults.resampleRector = resampleRactor;
MkcfResults.winRect = winRect;
% MkcfResults.normalizedWinRectSize = normalizedWinRectSize;
% MkcfResults.winRectY_f = winRectY_f;
% MkcfResults.normalizedScaleRectSize = normalizedScaleRectSize;
% MkcfResults.y_scale_f = y_scale_f;
MkcfResults.occBigRect = occBigRect;

MkcfResults.winRectSamples = winRectSamples;
MkcfResults.winRectAlpha_f = winRectAlpha_f;
MkcfResults.winRectD = winRectD;

MkcfResults.scaleSamples = scaleSamples;
MkcfResults.scaleAlpha_f = scaleAlpha_f;
MkcfResults.scaleD = scaleD;

MkcfResults.realRect = realRect;

end
