function [positions, fps, meanOverlap] = tracker(params)

close all;
clear global validChannelIdx;
clear global paddingParams;
clear global averageImageMap;
clear global net;

global enableGPU
enableGPU = true;
clear global normalAverageImage
% clear global occAverageImage
clear mex;

% addpath('external/matconvnet/matlab');
% addpath('external/matconvnet/matlab/mex');
% addpath('external/matconvnet/matlab/xtest');

global net

lambda1 = params.lambda1;
lambda2 = params.lambda2;
eta = params.eta;
scaleSensitivity = params.scaleSensitivity;
outputSigmaFactor = params.outputSigmaFactor;
cellSize = params.cellSize;
paddingSets = params.paddingSets;
visualization = params.visualization;
scaleDetection = params.scaleDetection;
outputIteration = params.outputIteration;

allLayers = [28, 37];
winRectLayer = [28, 37];

[imgPaths, gt, frameNum] = loadVideoInfo(videoPath, seqName, beginFrame);
% gt = ad_gt(gt);
positions = zeros(frameNum, 4);
if isempty(net)
    initial_net();
end

time = 0;
for frame = beginFrame : beginFrame + frameNum - 1;
    disp(['baseline_high  seq:' seqName ' frame:' num2str(frame)]);
    img = imread(imgPaths{frame - beginFrame + 1});
    
    tic;
    if size(img, 3) == 1
        img = repmat(img, [1,1,3]);
    end
    
    if frame == beginFrame
        objRect = [gt(1,2), gt(1,1), gt(1,4), gt(1,3)];
        imgSize = size(img);
        if sqrt(prod(objRect(3:4))) >= 150 || prod(imgSize(1:2)) > 400000
            resampleRactor = 0.5;
        else
            resampleRactor = 1;
        end
        
        objRect = floor(objRect * resampleRactor);
        init_paddingParams(paddingSets, objRect(3:4), size(img));
        
        % winRect
        winRect = changeRectPadding(objRect, 'originalRect', 'winRect');
        initWinRect = winRect;
        if sqrt(prod(initWinRect(3:4))) >= 15 * cellSize
            normalizedWinRectSize = initWinRect(3:4);
        else
            normalizedWinRectSize = initWinRect(3:4);
        end
        outputSigma = sqrt(prod(objRect(3:4))) * outputSigmaFactor / cellSize;
        winRectY_f = fft2(gaussian_shaped_labels(outputSigma, floor(normalizedWinRectSize / cellSize)));
        
        
        % scaleRect
        if scaleDetection == 1
            scaleWinRect = changeRectPadding(winRect, 'winRect', 'scaleRect');
            if sqrt(prod(scaleWinRect(3:4))) >= 15 * cellSize
                normalizedScaleRectSize = scaleWinRect(3:4);
            else
                normalizedScaleRectSize = scaleWinRect(3:4);
            end

            scaleOutputSigma = sqrt(prod(objRect(3:4))) / 15 / cellSize;
            y_scale_f = fft2(fftshift(gaussian_shaped_labels(scaleOutputSigma, floor(normalizedScaleRectSize / cellSize))));
        end
    end
    
    if resampleRactor < 1
        img = imresize(img, resampleRactor);
    end
    imgSize = size(img);
    
    
    if frame > beginFrame
              
        detectType = 'normalDetect';
        cnnFeatureRect = calcCnnFeatureRect(occBigRect, winRect, detectType);
        cnnFeature = getCnnFeature(img, cnnFeatureRect, allLayers);
        
        winRectPatchs = extractPatchFromCnnFeature(cnnFeature, winRectLayer, winRect, normalizedWinRectSize, cellSize, 1, 'hannwin', []);
        resizeRatio = sqrt(prod(winRect(3:4)) / prod(normalizedWinRectSize));
        response = detect(winRectSamples, winRectAlpha_f, winRectD, winRectPatchs, 'linear', []);
        
%         figure(2);
%         mesh(fftshift(response));
        
        sideLength = ceil(sqrt(prod(normalizedWinRectSize / cellSize)) * outputSigmaFactor * 2);
        [candPoints, maxR, maxC] = findCandPoints(response, sideLength);


        if  outputIteration == 1
            sz = floor(normalizedWinRectSize / cellSize);
            ky = circshift(-floor((sz(1) - 1)/2) : ceil((sz(1) - 1)/2), [1, -floor((sz(1) - 1)/2)]);
            kx = circshift(-floor((sz(2) - 1)/2) : ceil((sz(2) - 1)/2), [1, -floor((sz(2) - 1)/2)])';
            [bestDr, bestDc, ~] = resp_newton(response, fft2(response), 5, ky, kx, sz, maxR, maxC);
            winRect(1:2) = winRect(1:2) + round(resizeRatio * cellSize * [bestDr, bestDc]);
        else
            [bestDr, bestDc] = correctLoc(maxR, maxC, size(response));
            winRect(1:2) = winRect(1:2) + round(resizeRatio * cellSize * [bestDr - 1, bestDc - 1]);
        end
        
        if scaleDetection == 1 && mod(frame, 2) == 1 && (frame - beginFrame) >= 5
            winRect = searchScale(img, winRect, scaleSamples, scaleAlpha_f, scaleD, initWinRect, normalizedScaleRectSize, scaleSensitivity, cellSize);
        end
    end
    
    if prod(winRect(3:4)) / (paddingSets.padding.generic ^ 2) >= prod(imgSize(1:2))
        winRect = changeRectPadding(winRect, [0.5, 0.5]);
    end
    
    % training
    occBigRect = changeRectPadding(winRect, 'winRect', 'occBigRect');
    trainType = 'normalTrain';
    
    cnnFeatureRect = calcCnnFeatureRect(occBigRect, winRect, trainType);
    cnnFeature = getCnnFeature(img, cnnFeatureRect, allLayers);
    
    % winRectSample
    winRectPatchs = extractPatchFromCnnFeature(cnnFeature, winRectLayer, winRect, normalizedWinRectSize, cellSize, 1, 'hannwin', []);
    [newWinRectAlpha_f, newWinRectD] = trainModel(winRectPatchs, winRectY_f, lambda1, lambda2, 0.03, 'linear', []);
    if frame == beginFrame
        winRectSamples = winRectPatchs;
        winRectAlpha_f = newWinRectAlpha_f;
        winRectD = newWinRectD;
    else
        if exist('occFlag', 'var') && occFlag == 1
            eta_ad = eta * 6;
        else
            eta_ad = eta;
        end
        winRectSamples = updateSamples(winRectSamples, winRectPatchs, eta_ad);
        [winRectAlpha_f, winRectD] = updateMod(winRectAlpha_f, winRectD, newWinRectAlpha_f, newWinRectD, eta_ad);
    end
    disp(['winRectD: ' num2str(winRectD(1)) ' ' num2str(winRectD(2))]);
    
    % scaleSample
    if scaleDetection == 1
        scaleRect = changeRectPadding(winRect, 'winRect', 'scaleRect');   
        scalePatchs = extractScaleFeature(img, 'both', scaleRect, normalizedScaleRectSize, cellSize, 9, 1, 'hannwin', []);
        [newScaleAlpha_f, newScaleD] = trainModel(scalePatchs, y_scale_f, 0.01, lambda2, 0.03, 'gaussian', []);
        if frame == beginFrame
            scaleSamples = scalePatchs;
            scaleAlpha_f = newScaleAlpha_f;
            scaleD = newScaleD;
        else
            if exist('occFlag', 'var') && occFlag == 1
                eta_ad = eta * 6;
            else
                eta_ad = eta;
            end
            scaleSamples = updateSamples(scaleSamples, scalePatchs, eta_ad * 1);
            [scaleAlpha_f, scaleD] = updateMod(scaleAlpha_f, scaleD, newScaleAlpha_f, newScaleD, eta_ad * 1);
        end
    end
    
    
    occFlag = 0;
    
    objRect = changeRectPadding(winRect, 'winRect', 'originalRect');
    realRect = objRect ./ resampleRactor;
    positions(frame - beginFrame + 1, :) = realRect;
    time = time + toc;
    
    if visualization == 1
        if frame == beginFrame
            figure('NumberTitle','off', 'Name', ['Tracker - ' seqName]);
            imHandle = imagesc(uint8(img));
            rectHandle1 = rectangle('Position',[objRect(2), objRect(1), objRect(4), objRect(3)],'Curvature',[0,0],'EdgeColor','r', 'LineWidth', 3);
            rectHandle2 = rectangle('Position',[winRect(2), winRect(1), winRect(4), winRect(3)],'Curvature',[0,0],'EdgeColor','y', 'LineWidth', 3);
            rectHandle3 = rectangle('Position',[occBigRect(2), occBigRect(1), occBigRect(4), occBigRect(3)],'Curvature',[0,0],'EdgeColor','b', 'LineWidth', 3);
            textHandle = text(30,30,['\fontsize{30}\color{yellow}',['#', num2str(frame)]]);
            set(gca,'position',[0 0 1 1]); 
        else
            set(imHandle, 'CData', img);
            set(rectHandle1, 'Position', [objRect(2), objRect(1), objRect(4), objRect(3)]);
            set(rectHandle2, 'Position', [winRect(2), winRect(1), winRect(4), winRect(3)]);
            set(rectHandle3, 'Position', [occBigRect(2), occBigRect(1), occBigRect(4), occBigRect(3)]);
            set(textHandle, 'string', ['\fontsize{30}\color{yellow}',['#', num2str(frame)]]);
        end
        drawnow;
    end
end
positions = [positions(:,2), positions(:,1), positions(:,4), positions(:,3)];
[~, ~, meanOverlap] = calcOverlap(positions, gt);
fps = frameNum / time;
end

function [dr, dc] = correctLoc(dr, dc, responseSz)
if dr > responseSz(1) / 2
    dr = dr - responseSz(1);
end
if dc > responseSz(2) / 2
    dc = dc - responseSz(2);
end
end
