function [alpha_f, d] = trainModel(Samples, y_f, lambda1, lambda2, threshold, kernelType, sigma)
if length(Samples) >= 2 && nargin < 5
    error('Not enough params for model training.')
end
switch length(Samples)
    case 1
        alpha_f = trainSingleKernelModel(Samples{1}, y_f, lambda1, kernelType, sigma);
        d = [];
    case 2
        [alpha_f, d] = trainTwoKernelModel(Samples{1}, Samples{2}, y_f, lambda1, lambda2, threshold, kernelType, sigma);
    case 3
        [alpha_f, d] = trainMultiKernelModel(Samples, y_f, lambda1, lambda2, threshold, kernelType, sigma);
    otherwise
        error('Too many samples for model training.')
end

end

function alpha_f = trainSingleKernelModel(sample, y_f, lambda1, kernelType, sigma)
    k = calcK(sample.feat, sample.feat, kernelType, sigma);
    alpha_f = trainAlpha_f(k, k, [0.5; 0.5], y_f, lambda1);
end


function [alpha_f, d] = trainTwoKernelModel(sample1, sample2, y_f, lambda1, lambda2, threshold, kernelType, sigma)
k1 = calcK(sample1.feat, sample1.feat, kernelType, sigma);
k2 = calcK(sample2.feat, sample2.feat, kernelType, sigma);
% prevD = [0.5; 0.5];
% count = 0;
% stop = 0;
% while (stop == 0)
%     alpha_f = trainAlpha_f(k1, k2, prevD, y, lambda1);
%     alpha = ifft2(alpha_f);
%     d = trainD(k1, k2, alpha, y, lambda1, lambda2, prevD);
%     count = count + 1;
%     if (count > 1)
%         deltaAlpha = abs(alpha - prevAlpha);
%         deltaD = abs(d - prevD);
%         if (sum(deltaAlpha(:)) <= threshold * sum(abs(prevAlpha(:))) && sum(deltaD(:)) <= threshold * sum(abs(prevD(:))))
%             stop = 1;
%         end
%     end
%     prevAlpha = alpha;
%     prevD = d;
%     if (count >= 4)
%         break;
%     end
% end

if true %if sum(d < 0) > 0
    d1 = [0.25; 0.75];  d2 = [0.75; 0.25];
    alpha_f1 = trainAlpha_f(k1, k2, d1, y_f, lambda1);
    alpha_f2 = trainAlpha_f(k1, k2, d2, y_f, lambda1);
    y = ifft2(y_f);
    F1 = calcF(y, d1, {k1, k2}, alpha_f1, lambda1, lambda2);
    F2 = calcF(y, d2, {k1, k2}, alpha_f2, lambda1, lambda2);
    if (F1 <= F2)
        d = d1;
        alpha_f = alpha_f1;
    else
        d = d2;
        alpha_f = alpha_f2;
    end
end
end


function [alpha_f, d] = trainMultiKernelModel(Samples, y_f, lambda1, lambda2, threshold, kernelType, sigma)
kNum = length(Samples);
ks = cell(kNum, 1);
for i = 1:kNum
    ks{i} = calcK(Samples{i}.feat, Samples{i}.feat, kernelType, sigma);
end
% prevD = ones(kNum, 1) / kNum;
% count = 0;
% stop = 0;
% while (stop == 0)
%     alpha_f = trainMultiAlpha_f(ks, prevD, y, lambda1);
%     alpha = ifft2(alpha_f);
%     d = trainMultiD(ks, alpha, y, lambda1, lambda2, prevD);
%     count = count + 1;
%     if (count > 1)
%         deltaAlpha = abs(alpha - prevAlpha);
%         deltaD = abs(d - prevD);
%         if (sum(deltaAlpha(:)) <= threshold * sum(abs(prevAlpha(:))) && sum(deltaD(:)) <= threshold * sum(abs(prevD(:))))
%             stop = 1;
%         end
%     end
%     prevAlpha = alpha;
%     prevD = d;
%     if (count >= 4)
%         break;
%     end
% end

if true %if sum(d < 0) > 0
    d1 = [0.02; 0.1; 0.9];  d2 = [0.02; 0.9; 0.1];
    alpha_f1 = trainMultiAlpha_f(ks, d1, y_f, lambda1);
    alpha_f2 = trainMultiAlpha_f(ks, d2, y_f, lambda1);
    y = ifft2(y_f);
    F1 = calcF(y, d1, ks, alpha_f1, lambda1, lambda2);
    F2 = calcF(y, d2, ks, alpha_f2, lambda1, lambda2);
    if (F1 <= F2)
        d = d1;
        alpha_f = alpha_f1;
    else
        d = d2;
        alpha_f = alpha_f2;
    end
end
end