function patch = windowing(patch, validAreaRatio, winType, winParam)

global paddingParams

pSize = size(patch);
sz = floor(pSize(1:2) * validAreaRatio);

switch winType
    case 'hannwin'
        w = hann(sz(1)) * hann(sz(2))';
    case 'tukeywin'
        w = tukeywin(sz(1), winParam) * tukeywin(sz(2), winParam)';
    case 'gausswin'
        w = gausswin(sz(1), winParam) * gausswin(sz(2), winParam)';
    case 'gausswin_selfAdapt'
        r = paddingParams.occBigPadding ./ paddingParams.smallPadding;
        w = gausswin(sz(1), r(1)) * gausswin(sz(2), r(2))';
    otherwise
        error('Unknown "winType" in windowing');
end

if validAreaRatio == 1
    win = w;
else
    center = [1, 1] + floor(pSize(1:2) / 2);
    up = center(1) - floor(sz(1) / 2);
    down = up + sz(1) - 1;
    left = center(2) - floor(sz(2) / 2);
    right = left + sz(2) - 1;
    win = zeros(pSize(1), pSize(2));
    win(up:down, left:right) = w;
end
patch = bsxfun(@times, patch, win);
end